// Desc: A respawn selection dialog
// By: Dr Eyeball
//-----------------------------------------------------------------------------
// Constants
//-----------------------------------------------------------------------------
#define _spawnMarkerPattern "Respawn%1"
#define SPWN_BaseSafeZoneRadius 400

// The following IDD & IDC variables should match their equivalents for the dialog (from "Dialogs\RespawnDialog.hpp")
#define SPWN_IDD_RespawnDialog 389

#define SPWN_IDC_RespawnList 891
#define SPWN_IDC_RespawnMap 892
#define SPWN_IDC_AcceptButton 893
#define SPWN_IDC_Countdown 894

#define SPWN_RespawnLocationsCaption "Locations"
#define SPWN_ClickMarkerMaxDist 200 // distance in metres

#define SPWN_RespawnLocationMarker "m_RespawnLocation"

//-----------------------------------------------------------------------------
// Respawn Selection functions
//-----------------------------------------------------------------------------
SPWN_FillComboList = 
{
  private ["_idc", "_list", "_item", "_data", "_index"];
  _idc = _this select 0;
  _list = _this select 1;
  
  lbClear _idc;
  {
    _item = _x select 0;
    _data = _x select 1;
    _index = lbAdd [_idc, _item];
    lbSetData [_idc, _index, _data];
  } forEach _list;
  
  lbSetCurSel [_idc, 0];
};

//-----------------------------------------------------------------------------
// Dialog Events
//-----------------------------------------------------------------------------
SPWN_OnClose = 
{
  private ['_marker'];
  // hide marker
  SPWN_RespawnLocationMarker setMarkerShapeLocal "ICON";
  SPWN_RespawnLocationMarker setMarkerTypeLocal "Empty";

  _marker = lbData[SPWN_IDC_RespawnList, lbCurSel SPWN_IDC_RespawnList];
  if (_marker == "" || (markerPos _marker select 0 == 0 && markerPos _marker select 1 == 0)) then
  {
    _marker = format[_spawnMarkerPattern, playerSide];
    hint "Unsuitable location selected. Using base instead.";
  };

  //-----------------------------------
  //nul = [_marker] execVM "lkscripts\DScripts\Respawn\respawn_player.sqf"; // teleport

  titleText ["", "BLACK IN", 6];
  player setPos getMarkerPos _marker;
  //-----------------------------------

  // TODO: SPWN_RespawnRequested could be auto detected by checking (playerRespawnTime > 0)
  if (isNil "SPWN_RespawnRequested") then {SPWN_RespawnRequested = false};  
  if (SPWN_RespawnRequested) then // base deployment/teleport
  {
    SPWN_RespawnRequested = false;
  }
  else // respawn from dead
  {
    //[player] spawn fn_PersonInit;
  };
};
//-----------------------------------------------------------------------------
SPWN_AddSpawnPointMarker =
{
  private["_spawnPos", "_markerName", "_marker"];
  _spawnPos = _this select 0;
  
  _markerName = SPWN_RespawnLocationMarker;
  _marker = createMarkerLocal [_markerName, _spawnPos];
  //_markerName setMarkerShapeLocal "ELLIPSE";
  _markerName setMarkerShapeLocal "ICON";
  //_markerName setMarkerTypeLocal "";
  _markerName setMarkerTypeLocal "SELECT";
  _markerName setMarkerTextLocal "";
  _markerName setMarkerColorLocal "ColorBlack";
  //_markerName setMarkerSizeLocal [250, 250];
  _markerName setMarkerSizeLocal [0.6, 0.6];
  _markerName setMarkerPosLocal _spawnPos;
};
//-----------------------------------------------------------------------------
// Dialog Functions
//-----------------------------------------------------------------------------
SPWN_ViewSpawnPoint =
{
  private["_selection", "_control", "_selectedIndex", "_basePos", "_spawnMarkerName", "_index", 
    "_spawnPos", "_vehicleList", "_zoom"];
  _selection = _this select 0;  
  
  _control = _selection select 0;
  _selectedIndex = _selection select 1; 

  _basePos = getPos player;
  _spawnMarkerName = _control lbData _selectedIndex;
  
//player sidechat _spawnMarkerName; // debug

  /*
  _vehicleList = switch(playerSide)do
  {
    case west: {G_MobileRespawnVehicleListW};
    case east: {G_MobileRespawnVehicleListE};
    default {[]};
  };

  _index = [_spawnMarkerName, _vehicleList] call fn_GetParamIndexByName;
  _spawnPos = [0,0,0];
  if (_index == -1) then
  { 
  */  
    _spawnPos = getMarkerPos _spawnMarkerName; 
    _spawnPos set[2, 0.5]; // 0.5m above ground (above floor of some structures)
  /*
  }
  else
  {
    _spawnPos = getPos ((_vehicleList select _index) select 1); 
  };
  */

  //SPWN_RespawnLocation = _spawnMarkerName; // interferes
  
  if (typeName _spawnPos != "ARRAY") exitWith { /* 'invalid position type' */ }; // pos must be array
  if (count _spawnPos < 2) exitWith { /* 'invalid position value' */ }; // check if empty/invalid array returned
  if (_spawnPos select 0 == 0 && _spawnPos select 1 == 0) exitWith { /* 'invalid position value' */ }; // FOB/RP probably not deployed

  //---- show map location using panning & zooming
  ctrlMapAnimClear ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnMap); // reset while scrolling list
  _zoom = ctrlMapScale ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnMap);
  
  /*
  ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnMap) ctrlMapAnimAdd [0.1, _zoom, _basePos]; // show home base
  ctrlMapAnimCommit ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnMap);
  */
  
  ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnMap) ctrlMapAnimAdd [0.8, _zoom, _spawnPos]; // show map location
  ctrlMapAnimCommit ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnMap);
  
  /*
  ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnMap) ctrlMapAnimAdd [0.1, _zoom, _spawnPos]; // zoom map location
  ctrlMapAnimCommit ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnMap);
  */
  
  [_spawnPos] call SPWN_AddSpawnPointMarker;
};
//----------------------
SPWN_AcceptButton =
{
  private["_selectedIndex", "_marker"];
  _selectedIndex = lbCurSel SPWN_IDC_RespawnList;
  
  _marker = lbData [SPWN_IDC_RespawnList, _selectedIndex]; // assign marker name to global var
  //SPWN_RespawnLocation = _marker;
  if (_marker == "" || (markerPos _marker select 0 == 0 && markerPos _marker select 1 == 0)) exitWith
  {
    //SPWN_RespawnLocation = format[_spawnMarkerPattern, playerSide];
    hintC "Unsuitable location selected";
  };
  
  /*
  player globalChat format["Spawn location %1 selected. Marker name='%2'", 
    _selectedIndex, lbData [SPWN_IDC_RespawnList, _selectedIndex] ]; // debug
  */
    
  closeDialog 0;
};
//-----------------------------------------------------------------------------
SPWN_FindClickedSpawnPoint =
{
  private["_result", "_clickPos"];
  private["_i", "_list"];
  private["_clickDist", "_nearest", "_nearestDist"];

  _clickPos = _this select 0;

  _nearest = "";
  _nearestDist = 99000;
  
  _list = [];
  for [{_i = 0}, {_i < lbSize SPWN_IDC_RespawnList}, {_i = _i + 1}] do
  {
    _list = _list+[lbData [SPWN_IDC_RespawnList, _i]];
  };
  
  {
    _clickDist = (getMarkerPos _x) distance _clickPos;
    /*
    // check if click is within reasonable range of marker
    if (_clickDist < SPWN_ClickMarkerMaxDist) then
    {
    */
      // check if click is closest to this marker, if there are very close markers nearby
      if (_clickDist < _nearestDist) then
      {
        _nearest = _x;
        _nearestDist = _clickDist;
      };
    //};
  } forEach _list;

  if (_nearest != "") then
  {
    [getMarkerPos _nearest] call SPWN_AddSpawnPointMarker;
    lbSetCurSel[SPWN_IDC_RespawnList, _list find _nearest];
  };

  _nearest;
};
//-----------------------------------------------------------------------------
SPWN_Map_onMouseButtonClick =
{
  private["_eventParams", "_ctrl", "_button", "_x", "_y", "_shiftKey", "_ctrlKey", "_altKey"]; // event params
  private["_worldPos", "_noShift", "_marker"];
  
  _eventParams = _this select 0;
  
  _ctrl = _eventParams select 0;
  _button = _eventParams select 1;
  _x = _eventParams select 2;
  _y = _eventParams select 3;
  _shiftKey = _eventParams select 4;
  _ctrlKey = _eventParams select 5;
  _altKey = _eventParams select 6;
  //---------------------------------------------------------------------------
  _noShift = (!_shiftKey) && (!_ctrlKey) && (!_altKey);
  _worldPos = _ctrl ctrlMapScreenToWorld [_x, _y];
  _worldPos set[2, 0];

  // select spawn point
  if (_button == 0 && _noShift) then
  {
    /*_marker =*/ [_worldPos] call SPWN_FindClickedSpawnPoint;
  };
};
//-----------------------------------------------------------------------------
SPWN_SetDialogEvents =
{
  // init dialog events
  (findDisplay SPWN_IDD_RespawnDialog) displaySetEventHandler["Unload", "call SPWN_OnClose"];

  ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnMap)
    ctrlSetEventHandler ["MouseButtonClick", "[_this] call SPWN_Map_onMouseButtonClick"];

  ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnList)
    ctrlSetEventHandler ["LBSelChanged", "[_this] call SPWN_ViewSpawnPoint"];
  ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnList)
    ctrlSetEventHandler ["LBDblClick", "[_this] call SPWN_AcceptButton"];

  /*
  ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_AcceptButton)
    ctrlSetEventHandler ["ButtonClick", "[] call SPWN_AcceptButton"];
  */
};
//-----------------------------------------------------------------------------
SPWN_FillSpawnList =
{
  private['_currSelData', '_nestedArray', '_sSide', '_spawnMarkerText', '_marker', '_enemyList'];

  _nestedArray = [];
  _nestedArray = [_nestedArray] call compile preprocessFileLineNumbers "lkscripts\Dialogs\RespawnList.sqf";
  //-----------------------------------------------------------------------------
  if (str _nestedArray != str SPWN_SpawnList) then
  {
    SPWN_SpawnList = _nestedArray;

    ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnList)
      ctrlSetEventHandler ["LBSelChanged", ""]; // disable until end of function

    _currSelData = lbData[SPWN_IDC_RespawnList, lbCurSel SPWN_IDC_RespawnList];

    [SPWN_IDC_RespawnList, SPWN_SpawnList, false /* overwrite */] call SPWN_FillComboList;
    //---------------------
    // find & focus current respawn list item which triggers showing location marker
    _MatchingIndex = 0;
    _Size = lbSize SPWN_IDC_RespawnList;
    for [{_i = 0}, {_i < _Size}, {_i = _i + 1}] do
    {
      if (_currSelData == lbData [SPWN_IDC_RespawnList, _i]) then 
      {
        _MatchingIndex = _i;
      };
    };
    
    if (_Size > 0) then
    {
      lbSetCurSel [SPWN_IDC_RespawnList, _MatchingIndex];
    };
    //---------------------
    ((findDisplay SPWN_IDD_RespawnDialog) displayCtrl SPWN_IDC_RespawnList)
      ctrlSetEventHandler ["LBSelChanged", "[_this] call SPWN_ViewSpawnPoint"];
  };
  
  SPWN_RespawnLocationMarker setMarkerDirLocal ((markerDir SPWN_RespawnLocationMarker)+5); // animate marker for better visibility
};
//-----------------------------------------------------------------------------
SPWN_Start =
{
  private["_Params", "_status", "_color", "_index", "_MatchingIndex", "_Size", "_i", "_vehicleList", "_loc"];
  _Params = _this;

  call SPWN_SetDialogEvents;
  
  SPWN_SpawnList = [];
  call SPWN_FillSpawnList;
  //---------------------
  // find & focus current respawn location marker
  _MatchingIndex = 0;
  _Size = lbSize SPWN_IDC_RespawnList;
  if (isNil "SPWN_RespawnLocation") then {SPWN_RespawnLocation = ""};
  for [{_i = 0}, {_i < _Size}, {_i = _i + 1}] do
  {
    if (SPWN_RespawnLocation == lbData [SPWN_IDC_RespawnList, _i]) then 
    {
      _MatchingIndex = _i;
    };
  };
  
  if (_Size > 0) then
  {
    lbSetCurSel [SPWN_IDC_RespawnList, _MatchingIndex];
  };
  //---------------------
  // show the countdown time & refill list
  while {dialog && playerRespawnTime > -1} do
  {
    call SPWN_FillSpawnList;

    _timeRemaining = format["%1 - %2 seconds", SPWN_RespawnLocationsCaption, playerRespawnTime];
    ctrlSetText [SPWN_IDC_Countdown, _timeRemaining];

    sleep 1;
  };
  ctrlSetText [SPWN_IDC_Countdown, SPWN_RespawnLocationsCaption];
  //---------------------
  // refill list
  nul = [] spawn
  {
    while {dialog} do
    {
      call SPWN_FillSpawnList;
      sleep 1;
    };
  };
};
//-----------------------------------------------------------------------------
// Init
//-----------------------------------------------------------------------------
//if (dialog) exitWith {hintSilent "Another dialog is already open"; false};
if (!isNull findDisplay SPWN_IDD_RespawnDialog) exitWith {hintSilent "(Respawn dialog is already open)"};
_ok = createDialog "SPWN_RespawnDialog";
if !(_ok) exitWith { hint "createDialog failed" };

//SPWN_display = findDisplay SPWN_IDD_RespawnDialog;

_Params = _this;
[_Params] call SPWN_Start;

true // result
